pub enum LineSearchMethod {
MoreThuente {
c1: f64,
c2: f64,
width_tolerance: f64,
bounds: Array1<f64>,
},
HagerZhang {
delta: f64,
sigma: f64,
epsilon: f64,
theta: f64,
gamma: f64,
eta: f64,
bounds: Array1<f64>,
},
}Expand description
Line search methods for the local solver
This enum defines the types of line search methods that can be used in some of the local solver, including MoreThuente, HagerZhang, and Backtracking.
Variants